@@ -4,7 +4,7 @@
 namespace MDFN_IEN_NES
 {
 void FDSSoundReset(void) MDFN_COLD;
-bool FDS_SetMedia(uint32 drive_idx, uint32 state_idx, uint32 media_idx, uint32 orientation_idx) MDFN_COLD;
+void FDS_SetMedia(uint32 drive_idx, uint32 state_idx, uint32 media_idx, uint32 orientation_idx) MDFN_COLD;
 }
 
 #endif
