@@ -162,7 +162,7 @@ void InputDevice_Justifier::StateAction(StateMem* sm, const unsigned load, const
 
   SFVAR(command),
 
-  SFARRAY(transmit_buffer, sizeof(transmit_buffer)),
+  SFVAR(transmit_buffer),
   SFVAR(transmit_pos),
   SFVAR(transmit_count),
 
@@ -377,15 +377,15 @@ InputDevice *Device_Justifier_Create(void)
 
 IDIISG Device_Justifier_IDII =
 {
- { "x_axis", "X Axis", -1, IDIT_X_AXIS },
- { "y_axis", "Y Axis", -1, IDIT_Y_AXIS },
+ { "x_axis", "X Axis", -1, IDIT_POINTER_X },
+ { "y_axis", "Y Axis", -1, IDIT_POINTER_Y },
 
- { "trigger", "Trigger", 0, IDIT_BUTTON, NULL  },
+ IDIIS_Button("trigger", "Trigger", 0),
 
- { "o",	"O",		 1, IDIT_BUTTON,	NULL },
- { "start", "Start",	 2, IDIT_BUTTON,	NULL },
+ IDIIS_Button("o", "O", 1),
+ IDIIS_Button("start", "Start", 2),
 
- { "offscreen_shot", "Offscreen Shot(Simulated)", 3, IDIT_BUTTON, NULL },
+ IDIIS_Button("offscreen_shot", "Offscreen Shot(Simulated)", 3),
 };
 
 
