@@ -26,6 +26,9 @@
 #include "state-common.h"
 #include "Stream.h"
 
+namespace Mednafen
+{
+
 void MDFNSS_GetStateInfo(const std::string& path, StateStatusStruct* status);
 
 struct StateMem;
@@ -130,18 +133,16 @@ static INLINE SFORMAT SFBASE_(T* const v, const uint32 count, const char* const
 {
  return SFBASE_(v, count, 1, 0, v, name);
 }
-
+// Take care in how the SF*() macros are set up, or else stringification result of a macro passed as the "x" argument may change.
 #define SFVARN(x, ...)	SFBASE_(&(x), 1, __VA_ARGS__)
-
-#define SFVAR1_(x)	   SFVARN((x), #x)
-#define SFVAR4_(x, tc, rs, rb) SFVARN((x), tc, rs, rb, #x)
-#define SFVAR_(a, b, c, d, e, ...)	e
-#define SFVAR(...) 	SFVAR_(__VA_ARGS__, SFVAR4_, SFVAR3_, SFVAR2_, SFVAR1_, SFVAR0_)(__VA_ARGS__)
-
-#if SIZEOF_DOUBLE != 8
- #error "sizeof(double) != 8"
+#ifdef _MSC_VER
+ #define SFVAR(x, ...)   SFBASE_(&(x), 1, __VA_ARGS__, #x)
+#else
+ #define SFVAR(x, ...)   SFBASE_(&(x), 1, ## __VA_ARGS__, #x)
 #endif
 
+static_assert(sizeof(double) == 8, "sizeof(double) != 8");
+
 #define SFPTR8N(x, ...)		SFBASE_(SF_FORCE_A8(x), __VA_ARGS__)
 #define SFPTR8(x, ...)		SFBASE_(SF_FORCE_A8(x), __VA_ARGS__, #x)
 
@@ -183,4 +184,6 @@ static INLINE SFORMAT SFBASE_(T* const v, const uint32 count, const char* const
 //
 bool MDFNSS_StateAction(StateMem *sm, const unsigned load, const bool data_only, const SFORMAT *sf, const char *name, const bool optional = false) noexcept;
 
+}
+
 #endif
